#include "mmanager.h"
#include "../../file.h"
#include "../../exceptions.h"
#include "../../config.h"
#include "../../Attach_funcs.h"
#include <comutil.h>

Z80EX_BYTE C_MemoryManager::port7FFD;
BYTE C_MemoryManager::rom[0x8000];
BYTE C_MemoryManager::ram[0x4000*32];
BYTE* C_MemoryManager::rom_map;
BYTE* C_MemoryManager::ram_map;
bool C_MemoryManager::enable512;
extern C_Config config; 

void C_MemoryManager::ReadFile(void)
{
	int i;
	C_File  fl;
	wchar_t *fname;

	if (config.GetString(L"rom32k", &fname))
	{
		fl.Read(fname);
		fl.ReadBlock(rom, 0x8000);
		fl.Close();
	}
	else
	if (config.GetString(L"rom48", &fname))
	{
		fl.Read(fname);
		fl.ReadBlock(&rom[0x4000], 0x4000);
		fl.Close();

		if (config.GetString(L"rom128", &fname))
		{
			fl.Read(fname);
			fl.ReadBlock(rom, 0x4000);
			fl.Close();
		}
		else throw C_E(E_General, "(root/Roms/32k or (root/Roms/48 and root/Roms/128)) not found in config");
	}
	else throw C_E(E_General, "(root/Roms/32k or (root/Roms/48 and root/Roms/128)) not found in config");

	if (!config.GetBool("root/Enable512", &enable512)) enable512 = false;
}

void C_MemoryManager::Init(void)
{
	ReadFile();
	AttachZ80ReadHandler(OnReadByte);
	AttachZ80WriteHandler(OnWriteByte);
	AttachZ80OutputHandler(OnOutputByte);
	AttachResetHandler(OnReset);
	port7FFD = 0;
	Remap();
}

void C_MemoryManager::Close(void)
{
}

void C_MemoryManager::Remap(void)
{
	if (enable512) ram_map = &ram[(((port7FFD & 0xC0) >> 3) | (port7FFD & 7)) * 0x4000];
	else ram_map = &ram[(port7FFD & 7) * 0x4000];

	rom_map = &rom[(port7FFD & 16) ? 0x4000 : 0];
}

bool C_MemoryManager::OnReadByte(Z80EX_WORD addr, bool m1, Z80EX_BYTE &retval)
{
	if (addr < 0x4000) retval = rom_map[addr];
	else
	if (addr < 0x8000) retval = ram[addr-0x4000 + RAM_BANK5];
	else
	if (addr < 0xC000) retval = ram[addr-0x8000 + RAM_BANK2];
	else retval = ram_map[addr-0xC000];

	return true;
}

bool C_MemoryManager::OnWriteByte(Z80EX_WORD addr, Z80EX_BYTE value)
{
	if (addr < 0x4000) return false;

	if (addr < 0x8000) ram[addr-0x4000 + RAM_BANK5] = value;
	else
	if (addr < 0xC000) ram[addr-0x8000 + RAM_BANK2] = value;
	else ram_map[addr-0xC000] = value;

	return true;
}

bool C_MemoryManager::OnOutputByte(Z80EX_WORD port, Z80EX_BYTE value)
{
	if ((port & 0x8003) == 0x0001)	// 0x7FFD
	{
		if (!(port7FFD & 32))
		{
			if (enable512 && port!=0x7FFD) port7FFD = value & 7;
			else port7FFD = value;

			Remap();
		}
	}
	return false;
}

void C_MemoryManager::OnReset(void)
{
	port7FFD = 0;
	Remap();
}
